// $Id: pipe_win32.c,v 1.12 2005/05/25 12:33:38 lundman Exp $
// Win32 specific solutions to help with pipes.
// Jorgen Lundman January 29th, 2003.

#ifdef WIN32

// Universal system headers
#define _BSD_SIGNALS
#include <stdio.h>
#define WINDOWS_LEAN_AND_MEAN
#include <winsock2.h>
#include <io.h>
#include <errno.h>

#include "connections.h"
#include "lion.h"
#include "io.h"
#include "pipe.h"
#include "sockets.h"




// Right, here's the deal. We want to handle sockets, and files with one
// place of sleep/cpu-release. to be maximum efficient.
//
// But Windows don't allow file handles to be added to select().
// So we try WaitForMultipleObjects, and Overlapped IO
// but Win 95, 98 and ME don't handle Overlapped IO on disk files.
// There is no fcntl for nonblocking under windows.
// Right, seems the answer in Windows is to thread! Rather inefficient
// but if a user is running Windows they can't be all that concerned about
// efficiency.
// So we thread and create a pipe. But anonymous pipes are uni-directional.
// So we have to create a named pipe. 
// Named pipes are not supported under Win95, 98 and ME.
// So, lets call socketpair instead.
// There is no socketpair under winsock.
// So lets manually create a socketpair, thread off to deal with 
// file IO to is just written/read to pipe.
// If you open file O_RDWR it has to start two threads!
//
// And people think I'm biased because I love developing under Unix.
//
// Cos I'm assing tired of this, it is implemented in BLOCKING mode
// as it happens localhost connects are instant anyway, But in future
// this should be change to an internal thing, using lions own
// calls to set it all up
//

//
// Perform a fork(). The child gets NULL returned to it.
// If there is a failure, a valid node is still issues, and
// the _LOST event is sent. If the child actually needs to
// get a handle as well, it can pass the "child" ptrptr.
//




__RCSID("$LiON: lundman/lion/pipe_win32.c,v 1.12 2005/05/25 12:33:38 lundman Exp $");





connection_t *lion_filethread(int file, int (*start_address)(void *), void *arg, void *user_data)
{
	connection_t *parent_pipe, *child_pipe;
	SECURITY_ATTRIBUTES sec;
	HANDLE shared_mutex = NULL;

	parent_pipe = connections_new();
	
	parent_pipe->type = LION_TYPE_PIPE_FILE;
	parent_pipe->user_data = user_data;
	parent_pipe->file_socket = file;

	child_pipe = connections_new();
	
	child_pipe->type = LION_TYPE_PIPE;
	child_pipe->user_data = user_data; // child should call set_userdata.
	child_pipe->file_socket = file;


#ifdef DEBUG
	printf("[thread] making pipes\n");
#endif
	// Connect the two nodes.
	if (pipe_make_pipes(parent_pipe, child_pipe))
		goto failed;


	// Unfortunately, we need a mutex to stop the child from running right away.

	sec.nLength = sizeof(sec);
	sec.lpSecurityDescriptor = NULL;
	sec.bInheritHandle = TRUE;

#ifdef DEBUG
	printf("[thread] creating mutex\n");
#endif
	// Create the mutex to stop the child from running.
	shared_mutex = CreateMutex( &sec, TRUE, NULL );

	if (shared_mutex == NULL)
		goto failed;
	
	child_pipe->mutex = shared_mutex;
	parent_pipe->mutex = shared_mutex;

	// We don't want to consider the child node as parent anymore
	// The child sets this to disconnect when ready.
	// This would free this node far too soon.
	//child_pipe->type = LION_TYPE_NONE;
	lion_disable_read(child_pipe);


#ifdef DEBUG
	printf("[thread] creating thread\n");
#endif

	parent_pipe->thread = CreateThread(NULL, 0, 
									   (LPTHREAD_START_ROUTINE) start_address, 
									   (void *) child_pipe,0, NULL); 


	if ( parent_pipe->thread == NULL ) {
		goto failed;
	}



	// Parent!

	// We need to release the "child_pipe" on this side as we are parent.
	// and this node no longer exists on this end.


#ifdef DEBUG
	printf("[parent] setting pending. %p\n", parent_pipe);
#endif

	// since we may or may not have data for select, force a loop to 
	// send event.
	io_force_loop = 1;
	
	parent_pipe->status = ST_PENDING;
	return parent_pipe;

	

 failed:
	// We failed. Set parent status to DISCONNECT, and release the child.
	// lnet will then issue _LOST to application
	child_pipe->status = ST_DISCONNECT;
	lion_disconnect(child_pipe);

	parent_pipe->status = ST_PENDING;
	if (parent_pipe->socket > 0)
		sockets_close(parent_pipe->socket);
	parent_pipe->socket = -1;   // This will notify it of failure, WAIT? Who closes parent pipe?

	if (shared_mutex) {
		CloseHandle(shared_mutex);
		parent_pipe->mutex = NULL;
		child_pipe->mutex = NULL;
	}

	close( file );
	// We want select to poll next iteration and enter processing
	// as we don't have a real fd to add to select, but we need to send
	// events from that part of the library
	io_force_loop = 1;

	return NULL;

}



int pipe_child_starter(void *child_pipe)
{
	connection_t *chpipe = (connection_t *) child_pipe;
	connection_t *newd;
	
	// Sanity checks
	if (!chpipe ||
		!chpipe->start_address ||
		!chpipe->socket)
		_exit(1);

#ifdef DEBUG
	printf("[child_starter] waiting for mutex\n");
#endif

	if (chpipe->mutex) {
		WaitForSingleObject( chpipe->mutex, INFINITE );
		CloseHandle( chpipe->mutex );
		chpipe->mutex = NULL;
	}
	// We allocate a new node here which goes into "this" thread's
	// linked-list, then we tell the other thread to release its node.

	newd = connections_new();

	connections_dupe(newd, chpipe); // to, from


	// Release the other, which is in the main threads linked list.
	chpipe->socket = -1;
	chpipe->status = ST_DISCONNECT;
	chpipe->type = LION_TYPE_NONE;  // today
	

	newd->status = ST_PENDING;  // This way the child gets a connected event.
	newd->type = LION_TYPE_PIPE;

#ifdef DEBUG
	printf("[pipe_child_starter] calling child %p %p\n", chpipe, newd);
#endif

	ExitThread(newd->start_address(newd, newd->user_data));
	return 0; // NOT REACHED, BUT MSVC++ warning avoidance.
}




connection_t *lion_fork(int (*start_address)(connection_t *, void *), 
						int flags, void *user_data, void *arg)
{
	connection_t *parent_pipe, *child_pipe;
	SECURITY_ATTRIBUTES sec;
	HANDLE shared_mutex = NULL;

	parent_pipe = connections_new();
	
	parent_pipe->type = LION_TYPE_PIPE;
	parent_pipe->user_data = user_data;

	if (flags & LION_FLAG_TRACE)
		lion_enable_trace(parent_pipe);

	if (parent_pipe->trace)
		fprintf(trace_file, "%p: lion_fork\n", parent_pipe);

	child_pipe = connections_new();
	
	child_pipe->type = LION_TYPE_PIPE;
	child_pipe->user_data = user_data; // child should call set_userdata.

#ifdef DEBUG
	printf("[thread] making pipes\n");
#endif
	// Connect the two nodes.
	if (pipe_make_pipes(parent_pipe, child_pipe))
		goto failed;
 

	// Unfortunately, we need a mutex to stop the child from running right away.

	sec.nLength = sizeof(sec);
	sec.lpSecurityDescriptor = NULL;
	sec.bInheritHandle = TRUE;

#ifdef DEBUG
	printf("[thread] creating mutex\n");
#endif
	// Create the mutex to stop the child from running.
	shared_mutex = CreateMutex( &sec, TRUE, NULL );

	if (shared_mutex == NULL)
		goto failed;
	
	child_pipe->mutex = shared_mutex;
	parent_pipe->mutex = shared_mutex;

	// We don't want to consider the child node as parent anymore
	// The child sets this to disconnect when ready.
	child_pipe->type = LION_TYPE_NONE;
	child_pipe->start_address = start_address;

#ifdef DEBUG
	printf("[thread] creating thread\n");
#endif

	parent_pipe->thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) pipe_child_starter, (void *) child_pipe,0, NULL); //&a_threadId);


	if ( parent_pipe->thread == NULL ) {
		goto failed;
	}



	// Parent!

	// We need to release the "child_pipe" on this side as we are parent.
	// and this node no longer exists on this end.


#ifdef DEBUG
	printf("[parent] setting pending. %p\n", parent_pipe);
#endif

	// since we may or may not have data for select, force a loop to 
	// send event.
	io_force_loop = 1;
	
	parent_pipe->status = ST_PENDING;

	return parent_pipe;

	

 failed:

	// We failed, so set the reason here, so we can give it to the application
	parent_pipe->error_code = errno;
	
	// We failed. Set parent status to DISCONNECT, and release the child.
	// lnet will then issue _LOST to application
	child_pipe->status = ST_DISCONNECT;
	lion_disconnect(child_pipe);

	parent_pipe->status = ST_PENDING;
	if (parent_pipe->socket > 0)
		sockets_close(parent_pipe->socket);
	parent_pipe->socket = -1;   // This will notify it of failure, WAIT? Who closes parent pipe?

	if (shared_mutex) {
		CloseHandle(shared_mutex);
		parent_pipe->mutex = NULL;
		child_pipe->mutex = NULL;
	}

	// We want select to poll next iteration and enter processing
	// as we don't have a real fd to add to select, but we need to send
	// events from that part of the library
	io_force_loop = 1;

	return NULL;

}




connection_t *lion_execve(char *base, char **argv, char **envp,
						  int with_stderr, int lion_flags, void *user_data)
{
	connection_t *parent_pipe, *child_pipe;
	SECURITY_ATTRIBUTES sec;
	STARTUPINFO    si;
	PROCESS_INFORMATION  pi;

	parent_pipe = connections_new();
	
	parent_pipe->type = LION_TYPE_PIPE;
	parent_pipe->user_data = user_data;

	if (lion_flags & LION_FLAG_TRACE)
		lion_enable_trace(parent_pipe);

	child_pipe = connections_new();
	
	child_pipe->type = LION_TYPE_PIPE;
	child_pipe->user_data = user_data; // child should call set_userdata.

#ifdef DEBUG
	printf("[execv] making pipes\n");
#endif

	// Connect the two nodes.
	if (pipe_make_pipes(parent_pipe, child_pipe))
		goto failed;
 

	// Unfortunately, we need a mutex to stop the child from running right away.

	sec.nLength = sizeof(sec);
	sec.lpSecurityDescriptor = NULL;
	sec.bInheritHandle = TRUE;

	GetStartupInfo(&si);
    si.dwFlags    = STARTF_USESTDHANDLES;

	si.hStdInput  = child_pipe;
    si.hStdOutput = child_pipe;
    si.hStdError  = with_stderr ? child_pipe : GetStdHandle(STD_OUTPUT_HANDLE);

	// We don't want to consider the child node as parent anymore
	// The child sets this to disconnect when ready.

#ifdef DEBUG
	printf("[execv] creating process\n");
#endif

       if (!CreateProcess(
				NULL,
                argv[0],            // command line
                NULL,            // process security
                NULL,            // thread security
                TRUE,            // inherit handles-yes
                0,               // creation flags
                envp,            // environment block
                NULL,            // current directory
                &si,             // startup info
                &pi))            // process info (out)
		{
			goto failed;
		}


	parent_pipe->thread = pi.hProcess;
	parent_pipe->pid = pi.dwProcessId;
	// Parent!
	child_pipe->socket = -1; // The new child closes it
	child_pipe->type = LION_TYPE_NONE;
	child_pipe->status = ST_DISCONNECT;

	// We need to release the "child_pipe" on this side as we are parent.
	// and this node no longer exists on this end.


#ifdef DEBUG
	printf("[parent] setting pending. %p\n", parent_pipe);
#endif

	// since we may or may not have data for select, force a loop to 
	// send event.
	io_force_loop = 1;
	
	parent_pipe->status = ST_PENDING;

	return parent_pipe;

	

 failed:

#ifdef DEBUG
	printf("[execv] failed %d/%d\n", errno, GetLastError());
#endif

	// We failed. Set parent status to DISCONNECT, and release the child.
	// lnet will then issue _LOST to application

	if (lion_flags & LION_FLAG_FULFILL) {
		
		//child_pipe->status = ST_DISCONNECT;
		// Close the actual socket, and set fd to -1, that way
		// we will send a FAILED event to the application.
		sockets_close(child_pipe->socket);
		child_pipe->socket = -1;


                //child_pipe->type = LION_TYPE_PIPE;
                child_pipe->error_code = ENOENT;
                child_pipe->socket = -1;
                child_pipe->status = ST_PENDING;



		// Parent side we just need to clean up quietly.
		parent_pipe->status = ST_PENDING;
		if (parent_pipe->socket > 0)
			sockets_close(parent_pipe->socket);
		parent_pipe->socket = -1;   // This will notify it of failure, WAIT? Who closes parent pipe?

		io_force_loop = 1;

		return child_pipe;
	}
	
	
	child_pipe->status = ST_DISCONNECT;
	lion_disconnect(child_pipe);

	parent_pipe->status = ST_PENDING;
	if (parent_pipe->socket > 0)
		sockets_close(parent_pipe->socket);
	parent_pipe->socket = -1;   // This will notify it of failure, WAIT? Who closes parent pipe?


	
	
	
	
	// We want select to poll next iteration and enter processing
	// as we don't have a real fd to add to select, but we need to send
	// events from that part of the library
	io_force_loop = 1;

	return NULL;

}



//
// Connect two handles with pipes
//
int      pipe_make_pipes(connection_t *node1, connection_t *node2)
{
	int fd[2];
	// This is assuming you have socketpair function, and it has the UNIX
	// domain!

	// In future, we could also implement this by using our own functions
	// to open a listen on localhost, and connect to it, and tie the
	// two connections together.
	if( sockets_socketpair(fd) < 0 ) return -1;

	node1->socket = fd[0];
	node2->socket = fd[1];

	return 0; // Failure

}


int pipe_get_status( connection_t *node)
{
	int r;

#ifdef DEBUG
	printf("win32 get status: %p\n", node->thread);
#endif

	// Ok we don't know it yet.
	if (node->thread) {
		GetExitCodeThread( node->thread, &r );
		printf("exit code %d\n", r);
		return r;
	}

	return -1;
}



void lion_want_returncode( connection_t *node)
{

	node->status = ST_WANTRETURN;

}



static char *argv[MAX_ARG];

char **parser(char *s)
{
        char *ar, *arg;
        int argc;
        static char name[1024];

        if (!s) return NULL;

        strncpy(name, s, sizeof(name));
        //name = strdup(s);
        ar   = name;


        // For each argument, build a list.
        for( argc = 0; 
                 (arg = misc_digtoken(&ar, " \t\r\n")) && (argc < MAX_ARG-1);
                 argc++ ) {

                printf("  '%s'\n", arg);

                argv[ argc ] = arg;

        }

        // null-terminate.
        argv[ argc ] = NULL;

#ifdef DEBUG
        printf("[parse] %d arguments\n", argc);
#endif

        return argv;

}



connection_t *lion_system(char *name_orig, int with_stderr,
                          int lion_flags, void *user_data)
{
        char **argv;
        connection_t *ret;
        
        // Since the user might have passed us a static-string, which we can't
        // modify, we need to make a copy of it, and release.
        if (!name_orig) return NULL;


        argv = parser( name_orig );


        ret = lion_execve(argv[0], argv, NULL,
						  with_stderr, lion_flags, user_data);


        return ret;

}




//
// Start a new process, leaving a pipe to the process.
// We fork, then exec a new process.
//
#if 0
connection_t *lion_execv(char *base, char **argv, int with_stderr,
						void *user_data)
{
	connection_t *parent = NULL; // the node that we give to parent!
	connection_t *child  = NULL;
	int i, err;

	// NULL means child.

//	if ((parent = lion_fork( &child, user_data ))) {
	{
		// Parent code!
		
		// Lets have a sneak peak, did the fork work at all?
		if (parent->socket == -1) {
			// fork failed - so we just exit, the event gets posted
			// later.

			return parent;

		}

		// fork worked (but the execv might still fail before parents
		// gets here! )
		
		return parent;

	}




	/* child code! */

	// Set stdin|out and possibly err to the pipe.

    dup2(child->socket, 0);
    dup2(child->socket, 1);

	if (with_stderr) {
		dup2(child->socket, 2);
	} else {
		i = open("/dev/null", O_WRONLY);
		dup2(i, 2);
		close(i);
	}
	close(child->socket);
	child->socket = 0;


	// Now attempt to start the new process
    
    err = execv(argv[0], argv);

	// If we reach this place, execv failed.
	
	child->status = ST_DISCONNECT; // dont post events
	lion_disconnect( child );       // release the only node left.

    _exit(err);

}



//#if 0



connection_t *lion_system(char *name_orig, int with_stderr,
						 void *user_data)
{
	int argc;
	char *cp, *argv[100]; 
	char *name;
	connection_t *ret;
	
	// Since the user might have passed us a static-string, which we can't
	// modify, we need to make a copy of it, and release.
	if (!name_orig) return NULL;

	name = strdup(name_orig);
	
	for (argc = 0, cp = name;; cp = NULL) {
		if (!(argv[argc++] = strtok(cp, " \t\n")))
			break;
		
		
		/*
		 * This could win the World Prize for aweful code! :)
		 */
		
		if (argv[argc-1][ strlen(argv[argc-1])+1 ] == '"') { 
			/* Ah next token has " to start with. */
			char *r;
			
			
			r = &(argv[argc-1][ strlen(argv[argc-1]) + 2 ]);
			
			cp = r;
			
			if (!(argv[argc++] = strtok(cp, "\"\t\n")))
				break;
			
			cp = NULL;
			
		}
	}
	
	if (!argv[argc - 1]) argc--;


	ret = lion_execv(argv[0], argv, with_stderr, user_data);

	free(name);

	return ret;

}









#endif



#endif

