/*
 * template example
 *
 * by Jorgen Lundman <lundman@lundman.net> 
 *
 * template that can be used for more serious application. Creates new
 * nodes for each connection etc. Listens on a socket, and just relays
 * traffic, like a simple chat room.
 * 
 * 24/01/2003 - epoch
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <signal.h>

#include "lion.h"

#include "object.h"



static int master_switch = 0;
static int server_port   = 57777;  // Set to 0 for any port.


// The master listening socket. This could be a node as well, but
// since we only have one we have it here. We will only get _NEW from
// this one socket.
static object_t *listen = NULL;





void exit_interrupt(void)
{

	master_switch = 1;

}






int main(int argc, char **argv)
{

	signal(SIGINT, exit_interrupt);
#ifndef WIN32
	signal(SIGHUP, exit_interrupt);
#endif


	printf("Initialising Network...\n");

	lion_init();
	lion_compress_level( 0 );

	printf("Network Initialised.\n");



	// Create an initial game
		
	printf("Initialising Socket...\n");

	listen = object_new();
	listen->type = OBJECT_TYPE_LISTEN;

	listen->handle = lion_listen(&server_port, NULL, LION_FLAG_NONE, 
								 (void *) listen);


	if (!listen) {
		
		printf("Socket Failed...\n");
		master_switch = 1;
		
	}
	
	
	printf("Listening on port %d...\n", server_port);


	while( !master_switch ) {

		lion_poll(0, 1);     // This blocks. (by choice, FYI).

	}
	printf("\n");


	printf("Releasing Socket...\n");
	if (listen) {

		if (listen->handle) {
			lion_disconnect(listen->handle);
			listen->handle = NULL;
		}

		object_free(listen);
		listen = NULL;
	}
	printf("Socket Released.\n");


	object_free_all();

	lion_free();

	printf("Network Released.\n");

	printf("Done\n");

	return 0; // Avoid warning

}
