#include <stdio.h>
#include <string.h>

#include "lion.h"

#include "object.h"
#include "command.h"

#include "say.h"




//
// This is called from our userinput.c if the type of object is unregistered
//
int reg_userinput( lion_t *handle, 
				   void *user_data, int status, int size, char *line)
{
	object_t *node = (object_t *) user_data;  // Get our object struct

	// We should _always_ have a object node here, since we are only called
	// from userinput.c if we did have one. But, for sanity....

	if (!node)
		return 0;




	// Check which event we got and process...

	switch( status ) {

		
	case LION_CONNECTION_LOST:
		printf("Connection '%p' (%s) was lost: %d:%s\n", handle, 
			   node->username,
			   size, line);

		say_all("%s connection lost: %d:%s\r\n", node->username, size, line);

		node->handle = NULL;
		object_free( node );

		break;
		
	case LION_CONNECTION_CLOSED:
		printf("Connection '%p' (%s) was gracefully closed.\n", handle,
			   node->username);

		say_all("%s disconnected.\r\n", node->username);

		node->handle = NULL;
		object_free( node );
		break;
		
		// We have a new connection, send the greeting:
		// This isn't called here. We leave it for debugging reasons.
	case LION_CONNECTION_CONNECTED:
		break;

	case LION_BUFFER_USED:
		break;

	case LION_BUFFER_EMPTY:
		break;

	case LION_INPUT:

		if (!strncasecmp("quit", line, 4)) {
			lion_close(handle);
			break;
		}

		if (!strncasecmp("who", line, 4)) {
			command_who(node);
			break;
		}

		say_all("<%s> %s\r\n", node->username, line);

		break; 

	case LION_BINARY:
		break;

	}



	return 0;

}


