#include <stdio.h>

#include "lion.h"

#include "object.h"

#include "userinput.h"



//
// This is the function called by lion library when there is an event.
// To demonstrate how to write code that calls different places depending on
// the type of the object node, we split the logic here.
//


int lion_userinput( lion_t *handle, 
					void *user_data, int status, int size, char *line)
{
	object_t *node = (object_t *) user_data;  // Get our object struct


	// Basically, if we have no user_data node, and there for no object node
	// we just exit this function. This should never happen, since we always
	// pass a valid node of our own. But just incase, instead of core dumping..

	if (!user_data)
		return 0;


	// Now, check the type of the object node, and call the appropriate
	// function.

	switch (node->type) {

	case OBJECT_TYPE_NONE:   // Unknown type, shouldn't happen, so do nothing.
		return 0;

	case OBJECT_TYPE_LISTEN: // Our master listening socket.
		return listen_userinput( handle, user_data, status, size, line);

	case OBJECT_TYPE_UNREGISTERED: // An "unregistered" connection.
		return unreg_userinput( handle, user_data, status, size, line);

	case OBJECT_TYPE_REGISTERED: // An "registered" connection.
		return reg_userinput( handle, user_data, status, size, line);

	}

	// NOT REACHED
	return 0; // Avoid warning.
}


