// $Id: timers.h,v 1.2 2005/12/29 13:41:20 lundman Exp $
// Timer events, and triggers.
// Jorgen Lundman December 26th, 2005

#ifndef TIMERS_H
#define TIMERS_H

#include <time.h>
#ifdef WIN32
#define WINDOWS_MEAN_AND_LEAN
#include <winsock2.h>
#endif

#include "lion.h"

/* Includes */


#define TIMERS_FLAG_ONESHOT     0  // default
#define TIMERS_FLAG_RELATIVE    0  // default

#define TIMERS_FLAG_ABSOLUTE    1
#define TIMERS_FLAG_REPEAT      2



#ifndef timercmp
#define timercmp(tvp, uvp, cmp)                                         \
        (((tvp)->tv_sec == (uvp)->tv_sec) ?                             \
            ((tvp)->tv_usec cmp (uvp)->tv_usec) :                       \
            ((tvp)->tv_sec cmp (uvp)->tv_sec))
#endif
#ifndef timeradd
#define timeradd(tvp, uvp, vvp)                                         \
        do {                                                            \
                (vvp)->tv_sec = (tvp)->tv_sec + (uvp)->tv_sec;          \
                (vvp)->tv_usec = (tvp)->tv_usec + (uvp)->tv_usec;       \
                if ((vvp)->tv_usec >= 1000000) {                        \
                        (vvp)->tv_sec++;                                \
                        (vvp)->tv_usec -= 1000000;                      \
                }                                                       \
        } while (/* CONSTCOND */ 0)
#endif
#ifndef timersub
#define timersub(tvp, uvp, vvp)                                         \
        do {                                                            \
                (vvp)->tv_sec = (tvp)->tv_sec - (uvp)->tv_sec;          \
                (vvp)->tv_usec = (tvp)->tv_usec - (uvp)->tv_usec;       \
                if ((vvp)->tv_usec < 0) {                               \
                        (vvp)->tv_sec--;                                \
                        (vvp)->tv_usec += 1000000;                      \
                }                                                       \
        } while (/* CONSTCOND */ 0)

#endif




/* Variables */


struct timers_struct {
	struct timeval when;
	unsigned long major;   // sec    OR  hour
	unsigned long minor;   // usec   OR  mins
	int flags;
	void (*callback)(struct timers_struct *, void *);
	void *userdata;
};

typedef struct timers_struct timers_t;

typedef void (*lion_timer_t)(struct timers_struct *, 
							 void *);





/* Functions */

int       timers_init            ( void );
void      timers_free            ( void );
timers_t *timers_newnode         ( void );
void      timers_freenode        ( timers_t * );
int       timers_add             ( timers_t * );
timers_t *timers_new             ( unsigned long, unsigned long, int flags,
								   lion_timer_t, void *);
int       timers_cancel          ( timers_t * );
void      timers_select          ( struct timeval *timeout);
void      timers_process         ( void );


#endif
